#ifndef COMMON_H__
#define COMMON_H__

#include <iostream>
#include <vector>
#include <string>
#include <map>
#include <sstream>
#include <iomanip>
#include <queue>
#include <ctime>
#include <cmath>
#include <sys/time.h>

using namespace std;

#define NUM_ENCAP_THREAD 1      // how many encapsulator threads do we spawn
#define POLL_INTERVAL 1000000   // sleep period if no data is available in usec

// logger helper macros - you can disable debug message completely using Makefile
// We try logcxx, but it did *not* work out
// the logger crashes everywhere due to memory corruption, especially in constructors
#ifdef VERBOSE  
#define LOG_DEBUG(fmt,args...) { fprintf(stderr, "DEBUG: "); fprintf(stderr, fmt, args); fprintf(stderr, "\n"); }
#define LOG_WARN(fmt,args...)  { fprintf(stderr, "WARN: "); fprintf(stderr, fmt, args); fprintf(stderr, "\n"); }
#define LOG_ERROR(fmt,args...) { fprintf(stderr, "ERROR: "); fprintf(stderr, fmt, args); fprintf(stderr, "\n"); }
// single argument version
#define LOG_DEBUG1(arg) { fprintf(stderr, "DEBUG: %s\n", arg); }
#define LOG_WARN1(arg) { fprintf(stderr, "WARN: %s\n", arg); }
#define LOG_ERROR1(arg) { fprintf(stderr, "ERROR: %s\n", arg); }
#else
#define LOG_DEBUG(fmt,args...)      
#define LOG_WARN(fmt,args...)       
#define LOG_ERROR(fmt,args...)      
#define LOG_DEBUG1(arg) // single argument version
#define LOG_WARN1(arg)  
#define LOG_ERROR1(arg) 
#endif

enum STATUS {OK, FAILED}; // used as return values

class Mtv;      // forward declaration to avoid cyclic includes..
extern Mtv mtv; // the global Mtv instance, defined in mtv.cpp

// class Common contains all utility functions
class Common {
public:
    static inline int round_up(double val) { return floor(val + 0.5); }
    static inline unsigned long long getCurTimeMicros() { 
        struct timeval tv;
        unsigned long long retval;
        gettimeofday(&tv , NULL);
        retval = (((unsigned long long)(tv.tv_sec) * 1000000) + tv.tv_usec);
        return retval;
    }
    static void hexDump(const void* in_buff, int len){ // debugging function
        if (len <= 0) return;
        int i=0,j=0,k=0;
        unsigned char* buff = (unsigned char*) in_buff;
        if(!buff) return;
        printf("\n*************************** Hex Dump ***************************\n");
        for(;i<len;i++){
            // Print Hex Stuff
            printf("%02X",buff[i]);
            // Print ascii at the end of the line, Line length is 16 Hex and 16 Characters
            if(i==len-1){
                // The End of the last Hex line
                for(k=0;k<16-(i%16);k++) printf("   ");
                for(;j<=i;j++){ // Only print printable ascii characters
                    if((buff[j]>=32)&&(buff[j]<=126)) printf("%c",buff[j]);
                    else printf(".");
                }   
            }else if(i%16 == 15){ // The End of a normal Hex line
                printf("   ");
                for(;j<=i;j++){ // Only print printable ascii characters
                    if((buff[j]>=32)&&(buff[j]<=126)) printf("%c",buff[j]);
                    else printf(".");
                }
                j=i+1;  
                printf("\n");
            }else{ // between Hex characters
                printf(".");
            }
        }
        printf("\n*******************************************************************\n");
    }
    // two methods for collection classes
    struct DereferenceLess {
        template <typename PtrType>
        bool operator()(PtrType pT1, PtrType pT2) const {
        return *pT1 < *pT2;
        }
    };
    struct DereferenceMore {
        template <typename PtrType>
        bool operator()(PtrType pT1, PtrType pT2) const {
        return *pT1 > *pT2;
        }
    };

};

#endif // COMMON_H__
