#ifndef CONF_H__
#define CONF_H__

#include "common.hpp"

class Conf {
    public: 
        Conf();
        virtual ~Conf();
        virtual int getDataLength() = 0;  // amount of memory required for seralization
        virtual int fillData(unsigned char *buf, int offset) = 0;
        virtual int readData(unsigned char *buf, int offset) = 0;
        virtual string toString() = 0; // print the current variable values
        virtual STATUS set() = 0; // console input method.

    protected:
        // methods to pack variables into byte array, return new offset
        int enumToByteArray(unsigned char *buf, int offset, uint32_t enumeration);  
        int uint8ToByteArray(unsigned char *buf, int offset, uint8_t uint8);  
        int uint16ToByteArray(unsigned char *buf, int offset, uint16_t uint16);  
        int uint32ToByteArray(unsigned char *buf, int offset, uint32_t uint32); 
        int stringToByteArray(unsigned char *buf, int offset, string str);  
        // methods to parse byte array into variable, return new offset
        int byteArrayToEnum(unsigned char *buf, int offset, uint32_t &enumeration);  
        int byteArrayToUint8(unsigned char *buf, int offset, uint8_t &uint8);  
        int byteArrayToUint16(unsigned char *buf, int offset, uint16_t &uint16);  
        int byteArrayToUint32(unsigned char *buf, int offset, uint32_t &uint32);  
        int byteArrayToString(unsigned char *buf, int offset, string &str);  
        // convert data type into string
        template <class T>
        static inline string stringify(const T x) {
            ostringstream o;
            if (!(o << x)) {  // we failed
                return string("unknown format");
            }
            return o.str();
        }
        // template specialization for uint8_t, need casting
        static inline string stringify(const uint8_t x) {
            ostringstream o;
            if (!(o << ((int) x))) {  // we failed
                return string("unknown format");
            }
            return o.str();
        }
    private:
};

#endif // CONF_H__
