
#include "confmgr.hpp"
#include <iostream>
#include <fstream>

// file format. First four bytes are reserved and are filled up with signature
// values "ABCDEFAB". This is following by four bytes for length of NetworkConf
// After NetcorkConf, comes the ProgramConf vector. It starts with a four bytes
// field with the number of ProgramConf in it....

// The constructor takes an initial configuration file name as input
ConfMgr::ConfMgr(string _filename) {
    lastfilename = _filename;
    revert(); // try to load the file, ignore any failure (file not exists)
}

ConfMgr::~ConfMgr() {

}

void ConfMgr::console() {
    char command;
    string filename;
    while (1) {  // FOREVER
        cout << "MTV Console Interface" << endl;
        cout << "p) print current settings" << endl;
        cout << "a) add a new program" << endl;
        cout << "c) change settings" << endl;
        cout << "s) save" << endl;
        cout << "n) save as (a new file)" << endl;
        cout << "l) load (a previously saved file)" << endl;
        cout << "r) revert to the last saved file" << endl;
        cout << "Command: ";
        cin >> command;
        switch (command) {
            default:
                cout << "Unknown command: " << command << ", try again" << endl; 
                break;
            case 'p':
                cout << "Printing current configurations" << endl;
                dump();
                break;
            case 'a':
                cout << "Adding a new program" << endl;
                addProgram();
                break;
            case 'c':  // main body of console, we first figure out which save(...) method to call
                cout << "Changing settings, n)etwork or p)rogram? (n/p) "; 
                cin >> command;
                if (command == 'n' || command == 'N') {
                    network.set(); 
                }
                else if (command == 'p' || command == 'P') {
                    unsigned int n;  // program number
                    if (programs.size() > 0) {
                        cout << "Program number? (0 -- " << (programs.size() - 1) << ") ";
                        cin >> n; cin.ignore();
                        if (n >= 0 && n < programs.size()) {
                            programs.at(n).set();
                        }
                    }
                    else cout << "Please create a new program first" << endl;
                }
                else cout << "Unknown choice: " << command << endl;
                break;
            case 's':
                save();
                break;
            case 'n':
                cout << "Save as new filename: ";
                getline(cin, filename);
                cout << "Save to configuration file: " << filename << " (Y/N)? ";
                cin >> command;
                if (command == 'y' || command == 'Y') {
                    cout << "Saving to file: " << filename << endl;
                    saveas(filename);
                }
                else cout << "File not saved." << endl;
                break;
            case 'l':
                cout << "Load a configuration file, filename: ";
                getline(cin, filename);
                cout << "Load from configuration file: " << filename << " (Y/N)? ";
                cin >> command;
                if (command == 'y' || command == 'Y') {
                    cout << "Loading to file: " << filename << endl;
                    saveas(filename);
                }
                else cout << "File not loaded." << endl;
                break;
            case 'r':
                cout << "Revert to configuration file: " << lastfilename << " (Y/N)? ";
                cin >> command;
                if (command == 'y' || command == 'Y') {
                    cout << "Reverting to file: " << lastfilename << endl;
                    revert();
                }
                else cout << "Aborting reverting to file: " << lastfilename << endl;
                break;
        }
    }
}

STATUS ConfMgr::saveas(string newfilename) {
    unsigned char uint32[4];
    ofstream out(newfilename.c_str(), ios::out | ios::binary);
    if(! out) {  
        cout << "cannot open output file: " << newfilename << endl;
        return FAILED;
    }
    // file headers
    uint32[0] = (SIGNATURE >> 24) & 0xFF; // signature
    uint32[1] = (SIGNATURE >> 16) & 0xFF;
    uint32[2] = (SIGNATURE >> 8) & 0xFF;
    uint32[3] = (SIGNATURE) & 0xFF;
    out.write((char *) uint32, sizeof(uint32)); 
    // network conf
    int memSize = network.getDataLength();
    uint32[0] = (memSize >> 24) & 0xFF; // network configuration size
    uint32[1] = (memSize >> 16) & 0xFF;
    uint32[2] = (memSize >> 8) & 0xFF;
    uint32[3] = (memSize ) & 0xFF;
    out.write((char *) uint32, sizeof(uint32)); 
    unsigned char* data = new unsigned char[memSize];
    int offset = 0; // offset of data byte array
    offset = network.fillData(data, offset);
    out.write((char *) data, memSize);
    free(data);
    // program conf
    int numProgram = programs.size();
    uint32[0] = (numProgram >> 24) & 0xFF; // signature
    uint32[1] = (numProgram >> 16) & 0xFF;
    uint32[2] = (numProgram >> 8) & 0xFF;
    uint32[3] = (numProgram) & 0xFF;
    out.write((char *) uint32, sizeof(uint32)); 
    for (int i = 0; i < numProgram; i++) {
        ProgramConf program = programs.at(i);
        memSize = program.getDataLength();
        uint32[0] = (memSize >> 24) & 0xFF; // program configuration size
        uint32[1] = (memSize >> 16) & 0xFF;
        uint32[2] = (memSize >> 8) & 0xFF;
        uint32[3] = (memSize ) & 0xFF;
        out.write((char *) uint32, sizeof(uint32)); 
        data = new unsigned char[memSize];
        offset = 0; // reset the data offset
        offset = program.fillData(data, offset);
        out.write((char *) data, memSize);
        free(data);
    }
    out.close();  // save is done.
    lastfilename = newfilename;
    return OK; 
}

STATUS ConfMgr::load(string newfilename) {
    unsigned char uint32_array[4];
    ifstream in(newfilename.c_str(), ios::in| ios::binary);
    if(! in) {  
        cout << "cannot open input file: " << newfilename << endl;
        return FAILED;
    }
    // check signature
    in.read((char *) uint32_array, sizeof(uint32_array));
    uint32_t signature = uint32_array[0] << 24 | uint32_array[1] << 16 | uint32_array[2] << 8 | uint32_array[3];
    if (signature != SIGNATURE) {
        cout << "mismatched file signature: " << newfilename << endl;
        return FAILED;
    }
    // get networkconf length
    in.read((char *) uint32_array, sizeof(uint32_array));
    uint32_t memSize = uint32_array[0] << 24 | uint32_array[1] << 16 | uint32_array[2] << 8 | uint32_array[3];
    // read networkconf
    unsigned char* data = new unsigned char[memSize];
    int offset = 0; // offset of data byte array
    in.read((char *)data, memSize);
    offset = network.readData(data, offset);
    free(data);
    // read program configurations
    in.read((char *) uint32_array, sizeof(uint32_array)); // number of program..
    uint32_t numProgram = uint32_array[0] << 24 | uint32_array[1] << 16 | uint32_array[2] << 8 | uint32_array[3];
    programs.clear();  // drop all current data
    for (unsigned int i = 0; i < numProgram; i++) {
        in.read((char *) uint32_array, sizeof(uint32_array));
        memSize = uint32_array[0] << 24 | uint32_array[1] << 16 | uint32_array[2] << 8 | uint32_array[3];
        ProgramConf program; // the program to be read
        data = new unsigned char[memSize];
        offset = 0; // reset the offset
        in.read((char *)data, memSize);
        offset = program.readData(data, offset);
        free(data);
        programs.push_back(program); // program is ready
    }
    in.close(); // read is done
    lastfilename = newfilename;
    return OK;
}

STATUS ConfMgr::save() {
    return saveas(lastfilename);
}

STATUS ConfMgr::revert() {
    return load(lastfilename);
}

STATUS ConfMgr::addProgram() {
    programs.push_back(ProgramConf()); 
    return OK;
}



void ConfMgr::dump() {
    cout << "Crrent Configuration File name: " << lastfilename << endl;
    cout << "Network Configuration: " << endl;
    cout << network.toString() << endl;
    cout << "Program Configuration: " << endl;
    for (unsigned int i = 0; i < programs.size(); i++) {
        cout << "\tProgram " << i << " Configuration:" << endl;
        cout << programs.at(i).toString() << endl;
    }
}
