// entry point of the base station program
#ifndef MTV_H__
#define MTV_H__

#include "common.hpp"
#include "timeslicer.hpp"
#include "encapsulator.hpp"
#include "transmitter.hpp"
#include "confmgr.hpp"
#include "burst.hpp" 

// our main class
class Mtv {
public:    
    Mtv();    
    virtual ~Mtv();

    // synchronized method
    // we implement polling for now, we should add conditional variable in the future
    void pushReq(Burst* burst);    // non-blocking
    Burst* popReq();               // blocking call, the caller will sleep for POLL_INTERVAL   
    void pushRdy(Burst* burst);    // non-blocking
    Burst* popRdy();               // blocking call, the caller will sleep for POLL_INTERVAL   

    // this function is called after the initialization is done,
    // this function spawns all threads in the system
    void run();

private:
    // The three main components in the system
    Timeslicer* timeslicer;
    Encapsulator* encapsulator[NUM_ENCAP_THREAD];
    Transmitter* transmitter;
    // the corresponding threads
    pthread_t tithread;
    pthread_t enthread[NUM_ENCAP_THREAD];
    pthread_t trthread;
    // the priority queues to chain these three threads together we store
    // pointers in these queues, the burst objects are created by the
    // timeslicer, and delete by the transmitter
    priority_queue<Burst*, vector<Burst*>, Common::DereferenceMore> reqQueue;  // the first queue
    priority_queue<Burst*, vector<Burst*>, Common::DereferenceMore> rdyQueue;  // the first queue
    // mutex
    pthread_mutex_t reqMutex;
    pthread_mutex_t rdyMutex;
    // configuration manager
    ConfMgr confMgr;
};

#endif // MTV_H__
