#ifndef NETWORKCONF_H__
#define NETWORKCONF_H__

// byte array format: (1) unsigned ints are packed in big-endian, (2) strings are null terminated, i.e., with a one-byte overlead, (3) enums are packed as uint32_t, and (4) data are packed sequentially as they were declared

#include "common.hpp"
#include "conf.hpp"

// some enums used by network settings
enum FRAMESIZE {FRAMESIZEFIRST = -1, ROW256, ROW512, ROW768, ROW1024, FRAMESIZELAST};
enum FEC {FECFIRST = -1, FECON, FECOFF, FECLAST};
enum AVGRATE {AVGRATEFIRST = -1, R16K, R32K, R64K, R128K, R256K, R512K, R1024K, AVGRATELAST};
enum LANGCODE {LANGCODEFIRST = -1, ENG, LANGCODELAST};
enum BANDWIDTH {BANDWIDTHFIRST = -1, MHZ8=0, MHZ7=1, MHZ5=3, BANDWIDTHLAST};
enum CONSTELLATION {CONSTELLATIONFIRST = -1, QPSK=0, QAM16=1, QAM64=2, CONSTELLATIONLAST};
enum HIERARCHY {HIERARCHYFIRST = -1, DISABLE=0, HIERARCHYLAST};
enum CODERATEHP {CODERATEHPFIRST = -1, CR2=0, CR3=1, CR4=2, CR6=3, CR8=4, CODERATEHPLAST};
enum CODERATELP {CODERATELPFIRST = -1, NA=0, CODERATELPLAST};
enum GUARDINTERVAL {GUARDINTERVALFIRST = -1, GI32=0, GI16=1, GI8=2, GI4=3, GUARDINTERVALLAST};
enum TRANSMITMODE {TRANSMITMODEFIRST = -1, TM2K=0, TM8K=1, TM4K=2, TRANSMITMODELAST};
enum SCHEDULER {SCHEDULERFIRST = -1, RR, P2OPT, DBS, SMS, SMSP, SIMU, SIMUP, GLATS, GLATSB, SCHEDULERLAST};
// give display strings
struct framesizemap: map<unsigned int, string> {
    framesizemap() {
        this->operator[](ROW256) = "256 Rows";
        this->operator[](ROW512) = "512 Rows";
        this->operator[](ROW768) = "768 Rows";
        this->operator[](ROW1024) = "1024 Rows";
    };
    ~framesizemap(){};
};
struct fecmap: map<unsigned int, string> {
    fecmap() {
        this->operator[](FECON) = "Enabled";
        this->operator[](FECOFF) = "Disabled";
    };
    ~fecmap(){};
};
struct avgratemap: map<unsigned int, string> {
    avgratemap() {
        this->operator[](R16K) = "16 Kbps";
        this->operator[](R32K) = "32 Kbps";
        this->operator[](R64K) = "64 Kbps";
        this->operator[](R128K) = "128 Kbps";
        this->operator[](R256K) = "256 Kbps";
        this->operator[](R512K) = "512 Kbps";
        this->operator[](R1024K) = "1024 Kbps";
    };
    ~avgratemap(){};
};
struct langcodemap: map<unsigned int, string> {
    langcodemap() {
        this->operator[](ENG) = "English";
    };
    ~langcodemap(){};
};
struct bandwidthmap: map<unsigned int, string> {
    bandwidthmap() {
        this->operator[](MHZ5) = "5 MHz";
        this->operator[](MHZ7) = "7 MHz";
        this->operator[](MHZ8) = "8 MHz";
    };
    ~bandwidthmap(){};
};
struct constellationmap: map<unsigned int, string> {
    constellationmap() {
        this->operator[](QPSK) = "QPSK";
        this->operator[](QAM16) = "16QAM";
        this->operator[](QAM64) = "64QAM";
    };
    ~constellationmap(){};
};
struct hierarchymap: map<unsigned int, string> {
    hierarchymap() {
        this->operator[](DISABLE) = "Not Supported";
    };
    ~hierarchymap(){};
};
struct coderatehpmap: map<unsigned int, string> {
    coderatehpmap() {
        this->operator[](CR2) = "1/2";
        this->operator[](CR3) = "2/3";
        this->operator[](CR4) = "3/4";
        this->operator[](CR6) = "5/6";
        this->operator[](CR8) = "7/8";
    };
    ~coderatehpmap(){};
};
struct coderatelpmap: map<unsigned int, string> {
    coderatelpmap() {
        this->operator[](NA) = "Not Supported";
    };
    ~coderatelpmap(){};
};
struct guardintervalmap: map<unsigned int, string> {
    guardintervalmap() {
        this->operator[](GI4) = "4";
        this->operator[](GI8) = "8";
        this->operator[](GI16) = "16";
        this->operator[](GI32) = "32";
    };
    ~guardintervalmap(){};
};
struct transmitmodemap: map<unsigned int, string> {
    transmitmodemap() {
        this->operator[](TM2K) = "2K Mode";
        this->operator[](TM4K) = "4K Mode";
        this->operator[](TM8K) = "8K Mode";
    };
    ~transmitmodemap(){};
};
struct schedulermap: map<unsigned int, string> {
    schedulermap() {
        this->operator[](RR) = "Round-Robin";
        this->operator[](P2OPT) = "P2OPT";
        this->operator[](DBS) = "DBS";
        this->operator[](SMS) = "SMS";
        this->operator[](SMSP) = "SMS Prime";
        this->operator[](SIMU) = "SIMU";
        this->operator[](SIMUP) = "SIMU Prime";
        this->operator[](GLATS) = "GLATS";
        this->operator[](GLATSB) = "GLATSB";
    };
    ~schedulermap(){};
};

// list all fields
enum NETWORKFIELD { NETWORKFIELDFIRST = -1, FNUMPROGRAM, FNETWORKNAME, FNETWORKID, FFRAMESIZE, FFEC, FMAXBURSTDURATION, FAVGRATE, FPLATFORMID, FPLATFORMNAME, FLANGCODE, FTRANSPORTSTREAMID, FINTSERVICEID, FFREQUENCY, FBANDWIDTH, FCONSTELLATION, FHIERARCHY, FCODERATEHP, FCODERATELP, FGUARDINTERVAL, FTRANSMITMODE, FRESERVEDPSIBITRATE, FOUTTSFILENAME, FSCHEDULER, NETWORKFIELDLAST};
// give display string for each field
struct networkfieldmap: map<unsigned int, string> {
    networkfieldmap() {
        this->operator[](FNUMPROGRAM) = "Number of Programs"; 
        this->operator[](FNETWORKNAME) = "Network Name"; 
        this->operator[](FNETWORKID) = "Network Id"; 
        this->operator[](FFRAMESIZE) = "Frame Size"; 
        this->operator[](FFEC) = "FEC"; 
        this->operator[](FMAXBURSTDURATION) = "Maximum Burst Duration"; 
        this->operator[](FAVGRATE) = "Average Rate"; 
        this->operator[](FPLATFORMID) = "Platform Id";
        this->operator[](FPLATFORMNAME) = "Platform Name"; 
        this->operator[](FLANGCODE) = "Language Code"; 
        this->operator[](FTRANSPORTSTREAMID) = "Transport Stream Id"; 
        this->operator[](FINTSERVICEID) = "INT Service Id"; 
        this->operator[](FFREQUENCY) = "Frequency"; 
        this->operator[](FBANDWIDTH) = "Bandwidth"; 
        this->operator[](FCONSTELLATION) = "Constellation"; 
        this->operator[](FHIERARCHY) = "Hierarchical Modulation"; 
        this->operator[](FCODERATEHP) = "High-Priority Code Rate"; 
        this->operator[](FCODERATELP) = "Low-Priority Code Rate"; 
        this->operator[](FGUARDINTERVAL) = "Guard Interval"; 
        this->operator[](FTRANSMITMODE) = "Transmission Mode"; 
        this->operator[](FRESERVEDPSIBITRATE) = "Reserved PSI Bit Rate"; 
        this->operator[](FOUTTSFILENAME) = "Output TS Filename"; 
        this->operator[](FSCHEDULER) = "Burst Scheduling Algorithm";
    };
    ~networkfieldmap(){};
};

class NetworkConf : public Conf {
    public: 
        uint8_t numProgram;
        string networkName;
        uint8_t networkId;
        FRAMESIZE framesize;
        FEC fec;
        uint16_t maxBurstDuration;
        AVGRATE avgrate;
        uint32_t platformId;
        string platformName;
        LANGCODE langCode;
        uint32_t transportStreamId;
        uint16_t intServiceId;  // INT service id...
        uint32_t frequency;
        BANDWIDTH bandwidth;
        CONSTELLATION constellation;
        HIERARCHY hierarchy;
        CODERATEHP codeRateHp;
        CODERATELP codeRateLp;
        GUARDINTERVAL guardInterval;
        TRANSMITMODE transmitMode;
        uint32_t reservedPsiBitRate;
        string outTsFilename;
        SCHEDULER scheduler;

        virtual int getDataLength();  // amount of memory required for seralization
        virtual int fillData(unsigned char *buf, int offset);
        virtual int readData(unsigned char *buf, int offset);
        virtual string toString();
        virtual STATUS set(); // console input method.
        
        // these functions were ported from the Timeslicer file
        int getNetBitrate();
        int getAvailableBitrate();
        int getNetTimePerByte();
        int getAvailableTimePerByte();
    private:
        framesizemap framesizemap_;
        fecmap fecmap_;
        avgratemap avgratemap_;
        langcodemap langcodemap_;
        bandwidthmap bandwidthmap_;
        constellationmap constellationmap_;
        hierarchymap hierarchymap_;
        coderatehpmap coderatehpmap_;
        coderatelpmap coderatelpmap_;
        guardintervalmap guardintervalmap_;
        transmitmodemap transmitmodemap_;
        schedulermap schedulermap_;
        networkfieldmap networkfieldmap_;

        // constant bitrate tables <-- see cpp file for details
        static const float BITRATES_5MHZ[];
        static const float BITRATES_7MHZ[];
        static const float BITRATES_8MHZ[];
};

#endif // NETWORKCONF_H__
