
#include "programconf.hpp"


// basic constructor, create *one* primary/base stream
// without initializing the settings in it.
// This is used before reading a previously saved conf file.
// XXX Other constructors, for enhancement layers and boot 
// train will be added later.
ProgramConf::ProgramConf() : bootTrain(NULL) {
    primaryTrains.push_back(TrainConf());
}

ProgramConf::~ProgramConf() {
    if (bootTrain != NULL) {delete bootTrain; bootTrain = NULL; }
}

// this function returns the number of bytes required by saving the program conf
int ProgramConf::getDataLength() {
    int length = 0;
    length += NUMBOOTTRAINSIZE;
    if (bootTrain != NULL) {
        length += TRAINCONFLENSIZE;             // length
        length += bootTrain->getDataLength();   // real data
    }
    length += NUMPRIMARYTRAINSIZE;
    for (uint8_t i = 0; i < primaryTrains.size(); i++) {
        length += TRAINCONFLENSIZE;                       //length
        length += primaryTrains.at(i).getDataLength();   // real data
    }
    return length;
}

int ProgramConf::fillData(unsigned char *buf, int offset) {
    // first byte == 0 if no boot train, == 1 otherwise
    if (bootTrain == NULL) buf[offset] = 0; else buf[offset] = 1; 
    offset += 1;
    if (bootTrain != NULL) {
        uint32_t trainSize = bootTrain->getDataLength();
        offset = uint32ToByteArray(buf, offset, trainSize);
        offset = bootTrain->fillData(buf, offset);
    }
    uint8_t noTrains = primaryTrains.size();
    offset = uint8ToByteArray(buf, offset, noTrains);
    for (uint32_t i = 0; i < primaryTrains.size(); i++) {
        uint32_t trainSize = primaryTrains.at(i).getDataLength();
        offset = uint32ToByteArray(buf, offset, trainSize);
        offset = primaryTrains.at(i).fillData(buf, offset);
    }
    return offset;
}


int ProgramConf::readData(unsigned char *buf, int offset) {
    if (buf[offset] == 0) {
        offset += 1;
        bootTrain = NULL;
    }
    else { // == 1
        offset += 1;
        uint32_t trainSize;
        offset = byteArrayToUint32(buf, offset, trainSize);
        offset = bootTrain->readData(buf, offset);
    }
    uint8_t noTrains = (uint8_t) buf[offset]; offset += 1;
    for (uint8_t i = 0; i < noTrains; i++) {
        uint32_t trainSize;
        offset = byteArrayToUint32(buf, offset, trainSize);
        offset = primaryTrains.at(i).readData(buf, offset);
    }
    return offset;
}

string ProgramConf::toString() {
    string ret = "";
    if (bootTrain == NULL) ret += "\tNo Boot Train\n";
    else ret += ("\t Boot Train:\n" + bootTrain->toString());
    for (uint32_t i = 0; i < primaryTrains.size(); i++) {
        ret += ("\tPrimary Train " + stringify(i));
        ret += (":\n" + primaryTrains.at(i).toString());
    }
    return ret;
}

STATUS ProgramConf::set() {
    uint32_t n; 
    cout << "Configure 0) boot train, 1) primary train, Choice? ";
    cin >> n; cin.ignore(); // to clear up the CR in buffer
    switch (n) {
        case 0:
            if (bootTrain != NULL) { 
                bootTrain->set();
            }
            else {
                cout << "No boot train exists" << endl;
            }
            break;
        case 1:
            cout << "Promary train numbfer ? (0 -- " << (primaryTrains.size() - 1) << ") ";
            cin >> n; cin.ignore();
            if (n >= 0 && n < primaryTrains.size()) primaryTrains.at(n).set();
    }
 
    return OK;
}
