
#include "trainconf.hpp"

// this function returns the number of bytes required by saving the train data
int TrainConf::getDataLength() {
    int length = 0;
    length += sizeof(serviceId);
    length += sizeof(uint32_t); // serviceType
    length += sizeof(programPid);
    length += sizeof(uint32_t); // streamType
    length += sizeof(streamPid);
    length += sizeof(uint32_t); // broadcastId
    length += (7); // assume idSelectorByte is fixed length
    length += sizeof(deltaT);
    length += sizeof(bitRate);
    length += (1 + serviceProviderName.length()); // one byte for null termination
    length += (1 + serviceName.length()); 
    length += sizeof(dmax);
                                           
    return length;
}

int TrainConf::fillData(unsigned char *buf, int offset) {
    offset = uint16ToByteArray(buf, offset, serviceId);
    offset = enumToByteArray(buf, offset, serviceType);
    offset = uint16ToByteArray(buf, offset, programPid);
    offset = enumToByteArray(buf, offset, streamType);
    offset = uint16ToByteArray(buf, offset, streamPid);
    offset = enumToByteArray(buf, offset, broadcastId);
    memcpy(buf + offset, idSelectorByte, 7); offset += 7;
    offset = uint16ToByteArray(buf, offset, deltaT);
    offset = uint32ToByteArray(buf, offset, bitRate);
    offset = stringToByteArray(buf, offset, serviceProviderName);
    offset = stringToByteArray(buf, offset, serviceName);
    offset = uint32ToByteArray(buf, offset, dmax);
    return offset;
}


int TrainConf::readData(unsigned char *buf, int offset) {
    uint32_t enumeration;
    offset = byteArrayToUint16(buf, offset, serviceId);
    offset = byteArrayToEnum(buf, offset, enumeration); serviceType = (SERVICETYPE) enumeration;
    offset = byteArrayToUint16(buf, offset, programPid);
    offset = byteArrayToEnum(buf, offset, enumeration); streamType = (STREAMTYPE) enumeration;
    offset = byteArrayToUint16(buf, offset, streamPid);
    offset = byteArrayToEnum(buf, offset, enumeration); broadcastId = (BROADCASTID) enumeration;
    memcpy(idSelectorByte, buf + offset, 7); offset += 7;
    offset = byteArrayToUint16(buf, offset, deltaT);
    offset = byteArrayToUint32(buf, offset, bitRate);
    offset = byteArrayToString(buf, offset, serviceProviderName);
    offset = byteArrayToString(buf, offset, serviceName);
    offset = byteArrayToUint32(buf, offset, dmax);
    return offset;
}

string TrainConf::toString() {
    string ret = "";
    ret += ("\t\t" + trainfieldmap_[FSERVICEID] + ": "); 
    ret += (stringify(serviceId) + "\n");
    ret += ("\t\t" + trainfieldmap_[FSERVICETYPE] + ": "); 
    ret += (servicetypemap_[serviceType] + "\n");
    ret += ("\t\t" + trainfieldmap_[FPROGRAMPID] + ": "); 
    ret += (stringify(programPid) + "\n");
    ret += ("\t\t" + trainfieldmap_[FSTREAMTYPE] + ": "); 
    ret += (streamtypemap_[streamType] + "\n");
    ret += ("\t\t" + trainfieldmap_[FSTREAMPID] + ": "); 
    ret += (stringify(streamPid) + "\n");
    ret += ("\t\t" + trainfieldmap_[FBROADCASTID] + ": "); 
    ret += (broadcastidmap_[broadcastId] + "\n");
    ret += ("\t\t" + trainfieldmap_[FIDSELECTORBYTE] + ": "); 
    // convert the id selector byte into hex
    stringstream hexStringStream;
    hexStringStream << hex << setfill('0');
    for(int index; index < 7; index++) 
        hexStringStream << setw(2) << static_cast<int>(idSelectorByte[index]);
    ret += (hexStringStream.str() + "\n");
    ret += ("\t\t" + trainfieldmap_[FDELTAT] + ": "); 
    ret += (stringify(deltaT) + "\n");
    ret += ("\t\t" + trainfieldmap_[FBITRATE] + ": "); 
    ret += (stringify(bitRate) + "\n");
    ret += ("\t\t" + trainfieldmap_[FSERVICEPROVIDERNAME] + ": "); 
    ret += (serviceProviderName + "\n");
    ret += ("\t\t" + trainfieldmap_[FSERVICENAME] + ": "); 
    ret += (serviceName + "\n");
    ret += ("\t\t" + trainfieldmap_[FDMAX] + ": ");
    ret += (stringify(dmax) + "\n");
    return ret;
}

STATUS TrainConf::set() {
    // iterate through the TRAINFIELD enum
    for (int i = TRAINFIELDFIRST + 1; i < TRAINFIELDLAST; i++) { 
        cout << i << ") " << trainfieldmap_[i] << endl;
    }
    uint32_t n;  // choice
    cout << "Choice? ";
    cin >> n; 
    cin.ignore(); // to clear up the CR in buffer
    cout << "Input " << trainfieldmap_[n] << ": "; 
    switch (n) {
        case FSERVICEID: cin >> serviceId; break;
        case FSERVICETYPE: cout << "(0) TV, (1) INT, Choice? "; cin >> n; cin.ignore(); if (n == 0) serviceType = TVSERVICE; else if (n == 1) serviceType = INTSERVICE; break;
        case FPROGRAMPID: cin >> programPid; break;
        case FSTREAMTYPE: cout << "(0) TV, (1) INT, Choice? "; cin >> n; cin.ignore(); if (n == 0) streamType = TVSTREAM; else if (n == 1) streamType = INTSTREAM; break;
        case FSTREAMPID: cin >> streamPid; break;
        case FBROADCASTID: cout << "(0) TV, (1) INT, Choice? "; cin >> n; cin.ignore(); if (n == 0) broadcastId = TVBROADCAST; else if (n == 1) broadcastId = INTBROADCAST; break;
        case FIDSELECTORBYTE: cout << "Enter seven hexadecimal bytes: " << endl; cin >> hex >> idSelectorByte; break;
        case FDELTAT: cin >> deltaT; break;
        case FBITRATE: cin >> bitRate; break;
        case FSERVICEPROVIDERNAME: getline(cin, serviceProviderName); break;
        case FSERVICENAME: getline(cin, serviceName); break;
        case FDMAX: cin >> dmax; break;
    }
    return OK;
}
