#ifndef TRAINCONF_H__
#define TRAINCONF_H__

// byte array format: (1) unsigned ints are packed in big-endian, (2) strings are null terminated, i.e., with a one-byte overlead, (3) selectorByte is handled in the same way as strings, (4) enums are packed as uint32_t, and (5) data are packed sequentially as they were declared

#include "common.hpp"
#include "conf.hpp"

// some enums used by network settings
enum SERVICETYPE {TVSERVICE = 0x01, INTSERVICE = 0x80};
enum STREAMTYPE {TVSTREAM = 0x90, INTSTREAM = 0x05};
enum BROADCASTID {TVBROADCAST = 0x0005, INTBROADCAST = 0x000B};
// give display strings
struct servicetypemap: map<unsigned int, string> {
    servicetypemap() {
        this->operator[](TVSERVICE) = "TV";
        this->operator[](INTSERVICE) = "INT";
    };
    ~servicetypemap(){};
};
struct streamtypemap: map<unsigned int, string> {
    streamtypemap() {
        this->operator[](TVSTREAM) = "TV";
        this->operator[](INTSTREAM) = "INT";
    };
    ~streamtypemap(){};
};
struct broadcastidmap: map<unsigned int, string> {
    broadcastidmap() {
        this->operator[](TVBROADCAST) = "TV";
        this->operator[](INTBROADCAST) = "INT";
    };
    ~broadcastidmap(){};
};


// list all fields
enum TRAINFIELD { TRAINFIELDFIRST = -1, FSERVICEID, FSERVICETYPE, FPROGRAMPID, FSTREAMTYPE, FSTREAMPID, FBROADCASTID, FIDSELECTORBYTE, FDELTAT, FBITRATE, FSERVICEPROVIDERNAME, FSERVICENAME, FDMAX, TRAINFIELDLAST};
// give display string for each field
struct trainfieldmap: map<unsigned int, string> {
    trainfieldmap() {
        this->operator[](FSERVICEID) = "Service Id";
        this->operator[](FSERVICETYPE) = "Service Type";
        this->operator[](FPROGRAMPID) = "Program Pid";
        this->operator[](FSTREAMTYPE) = "Stream Type";
        this->operator[](FSTREAMPID) = "Stream Pid";
        this->operator[](FBROADCASTID) = "Broadcast Id";
        this->operator[](FIDSELECTORBYTE) = "Id Selector Byte";
        this->operator[](FDELTAT) = "Inter-Burst Time";
        this->operator[](FBITRATE) = "Bit Rate";
        this->operator[](FSERVICEPROVIDERNAME) = "Service Provider Name";
        this->operator[](FSERVICENAME) = "Service Name";
        this->operator[](FDMAX) = "Allowable Channel Switching Delay";
    };
    ~trainfieldmap(){};
};

class TrainConf : public Conf {
    public: 
        uint16_t serviceId;
        SERVICETYPE serviceType;
        uint16_t programPid;
        STREAMTYPE streamType;
        uint16_t streamPid;
        BROADCASTID broadcastId;
        unsigned char idSelectorByte[7];    // assume id selector byte is 7 byte, 
                                            // see pp. 22 of 301 192 V.1.4.1 
        uint16_t deltaT;
        uint32_t bitRate;
        string serviceProviderName;
        string serviceName;
        uint32_t dmax;        

        virtual int getDataLength();  // amount of memory required for seralization
        virtual int fillData(unsigned char *buf, int offset);
        virtual int readData(unsigned char *buf, int offset);
        virtual string toString();
        virtual STATUS set(); // console input method.

    private:
        servicetypemap servicetypemap_;
        streamtypemap streamtypemap_;
        broadcastidmap broadcastidmap_;
        trainfieldmap trainfieldmap_;
};


#endif // TRAINCONF_H__
